local _G = _G
local SpeakUp = SpeakUp
local AceConfig = SpeakUp.LS("AceConfig-3.0")
local AceConfigDialog = SpeakUp.LS("AceConfigDialog-3.0")
local AceDBOptions = SpeakUp.LS("AceDBOptions-3.0")
local L = SpeakUp.L
local CreateFrame, ipairs = CreateFrame, ipairs

--First visible frame
local function createMainPanel()
	local frame = CreateFrame("Frame", "SpeakUpOptionsMain")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local version = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local author = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetFormattedText("|T%s:%d|t %s", "Interface\\GossipFrame\\PetitionGossipIcon", 16, "Speak Up")
	title:SetPoint("CENTER", frame, "CENTER", 0, 170)
	version:SetText(_G.GAME_VERSION_LABEL .. " " .. GetAddOnMetadata("SpeakUp", "Version"))
	version:SetPoint("CENTER", frame, "CENTER", 0, 130)
	author:SetText(L["Author"] .. ": Deepac")
	author:SetPoint("CENTER", frame, "CENTER", 0, 100)
	return frame
end

--event options frame
local optionsPanel = {
	order = 1,
	type = "group",
	name = L["Events"],
	inline = false,
	args = {
	},
}

--respond to Default button/Profile reset
function SpeakUp:profileReset() for _, e in ipairs(SpeakUp.EventList) do SpeakUp.frame:UnregisterEvent(e.event) end end

local function getEventValue(key) return SpeakUp.db.profile[key] end
local function setEventValue(event, value)
	SpeakUp.db.profile[event.key] = value
	if event.event == "COMBAT_LOG_EVENT" and value then
		local guid = UnitGUID("pet")
		CombatLogAddFilter("UNIT_DIED", guid)
	end
	if value then SpeakUp.frame:RegisterEvent(event.event)
	else SpeakUp.frame:UnregisterEvent(event.event) end
end

--Setup options
function SpeakUp:createOptions()
	local mainPanel = createMainPanel()
	mainPanel.name = "Speak Up"
	InterfaceOptions_AddCategory(mainPanel)
	--make sure Speak Up's reset sub is called in response to a profile reset
	SpeakUp.db.RegisterCallback(SpeakUp, "OnProfileReset", "profileReset")
	--build options
	for i, e in ipairs(SpeakUp.EventList) do
		if optionsPanel.args[e.category] == nil then
			optionsPanel.args[e.category] = {}
			optionsPanel.args[e.category].order = i
			optionsPanel.args[e.category].type = "group"
			optionsPanel.args[e.category].name = e.category
			optionsPanel.args[e.category].inline = false
			optionsPanel.args[e.category].args = {}
		end
		optionsPanel.args[e.category].args["opt" .. e.key] = {}
		optionsPanel.args[e.category].args["opt" .. e.key].order = e.order
		optionsPanel.args[e.category].args["opt" .. e.key].type = "toggle"
		optionsPanel.args[e.category].args["opt" .. e.key].width = "full"
		optionsPanel.args[e.category].args["opt" .. e.key].name = e.name
		optionsPanel.args[e.category].args["opt" .. e.key].get = function() return getEventValue(e.key) end
		optionsPanel.args[e.category].args["opt" .. e.key].set = function(info, value) setEventValue(e, value) end
		optionsPanel.args[e.category].args["snd" .. e.key] = {}
		optionsPanel.args[e.category].args["snd" .. e.key].order = e.order + 0.5
		optionsPanel.args[e.category].args["snd" .. e.key].type = "select"
		optionsPanel.args[e.category].args["snd" .. e.key].dialogControl = "LSM30_Sound"
		optionsPanel.args[e.category].args["snd" .. e.key].values = SpeakUp.LSM:HashTable(SpeakUp.LSM.MediaType.SOUND)
		optionsPanel.args[e.category].args["snd" .. e.key].name = _G.SOUND_LABEL
		optionsPanel.args[e.category].args["snd" .. e.key].get = function() return SpeakUp.db.profile[e.key .. "sound"] end
		optionsPanel.args[e.category].args["snd" .. e.key].set = function(info, value) SpeakUp.db.profile[e.key .. "sound"] = value end
	end
	AceConfig:RegisterOptionsTable("SpeakUpOptions", optionsPanel)
	AceConfig:RegisterOptionsTable("SpeakUpProfiles", AceDBOptions:GetOptionsTable(SpeakUp.db))
	SpeakUp.options = AceConfigDialog:AddToBlizOptions("SpeakUpOptions", _G.MAIN_MENU, "Speak Up")
	--make sure Speak Up's reset function is call in response to the user clicking the Default button
	SpeakUp.options.default = function() SpeakUp.db:ResetProfile() end
	AceConfigDialog:AddToBlizOptions("SpeakUpProfiles", L["Profiles"], "Speak Up")
end